@@ -38,10 +38,10 @@ import functools
 import logging
 from typing import Any, Callable, Dict, Mapping, Optional, Sequence, Tuple
 
+from absl import logging
 import frozendict
 import jax
 import jax.numpy as jnp
-import numpy as np
 import numpyro
 from numpyro import infer
 
@@ -76,6 +76,7 @@ class LightweightMMM:
   Attributes:
     trace: Sampling trace of the bayesian model once fitted.
     n_media_channels: Number of media channels the model was trained with.
+    n_geos: Number of geos for geo models or 1 for national models.
     model_name: Name of the model.
     media: The media data the model is trained on. Usefull for a variety of
       insights post model fitting.
@@ -96,8 +97,9 @@ class LightweightMMM:
       total_costs: jnp.ndarray,
       target: jnp.ndarray,
       extra_features: Optional[jnp.ndarray] = None,
-      degrees_seasonality: int = 3,
+      degrees_seasonality: int = 2,
       seasonality_frequency: int = 52,
+      weekday_seasonality: bool = False,
       media_names: Optional[Sequence[str]] = None,
       number_warmup: int = 1000,
       number_samples: int = 1000,
@@ -112,15 +114,18 @@ class LightweightMMM:
     respective function in the models.py file.
 
     Args:
-      media: Media input data.
+      media: Media input data. Media data must have either 2 dims for national
+        model or 3 for geo models.
       total_costs: Costs of each media channel. The number of cost values must
         be equal to the number of media channels.
       target: Target KPI to use, like for example sales.
       extra_features: Other variables to add to the model.
       degrees_seasonality: Number of degrees to use for seasonality. Default is
-        3.
+        2.
       seasonality_frequency: Frequency of the time period used. Default is 52 as
         in 52 weeks per year.
+      weekday_seasonality: In case of daily data, also estimate seven weekday
+        parameters.
       media_names: Names of the media channels passed.
       number_warmup: Number of warm up samples. Default is 1000.
       number_samples: Number of samples during sampling. Default is 1000.
@@ -134,12 +139,24 @@ class LightweightMMM:
       seed: Seed to use for PRNGKey during training. For better replicability
         run all different trainings with the same seed.
     """
+    if media.ndim not in (2, 3):
+      raise ValueError(
+          "Media data must have either 2 dims for national model or 3 for geo "
+          "models.")
+    if media.ndim == 3 and total_costs.ndim == 1:
+      total_costs = jnp.expand_dims(total_costs, axis=-1)
+
     if media.shape[1] != len(total_costs):
       raise ValueError("The number of data channels provided must match the "
                        "number of cost values.")
     if media.min() < 0:
       raise ValueError("Media values must be greater or equal to zero.")
 
+    if weekday_seasonality and seasonality_frequency == 52:
+      logging.warn("You have chosen daily seasonality and frequency 52 "
+                   "(weekly), please check you made the right seasonality "
+                   "choices.")
+
     if extra_features is not None:
       extra_features = jnp.array(extra_features)
 
@@ -165,13 +182,15 @@ class LightweightMMM:
         cost_prior=jnp.array(total_costs),
         degrees_seasonality=degrees_seasonality,
         frequency=seasonality_frequency,
-        transform_function=self._model_transform_function)
+        transform_function=self._model_transform_function,
+        weekday_seasonality=weekday_seasonality)
 
     if media_names is not None:
       self.media_names = media_names
     else:
       self.media_names = [f"channel_{i}" for i in range(media.shape[1])]
     self.n_media_channels = media.shape[1]
+    self.n_geos = media.shape[2] if media.ndim == 3 else 1
     self._total_costs = total_costs
     self.trace = mcmc.get_samples()
     self._number_warmup = number_warmup
@@ -181,6 +200,7 @@ class LightweightMMM:
     self._train_media_size = train_media_size
     self._degrees_seasonality = degrees_seasonality
     self._seasonality_frequency = seasonality_frequency
+    self._weekday_seasonality = weekday_seasonality
     self.media = media
     self._extra_features = extra_features
     self._mcmc = mcmc
@@ -195,11 +215,13 @@ class LightweightMMM:
   @functools.partial(
       jax.jit,
       static_argnums=(0,),
-      static_argnames=("degrees_seasonality", "transform_function", "model"))
+      static_argnames=("degrees_seasonality", "weekday_seasonality",
+                       "transform_function", "model"))
   def _predict(self, rng_key: jnp.ndarray, media_data: jnp.ndarray,
                extra_features: Optional[jnp.ndarray], cost_prior: jnp.ndarray,
                degrees_seasonality: int, frequency: int,
                transform_function: Callable[[Any], jnp.ndarray],
+               weekday_seasonality: bool,
                model: Callable[[Any], None],
                posterior_samples: Dict[str, jnp.ndarray]
                ) -> Dict[str, jnp.ndarray]:
@@ -215,6 +237,7 @@ class LightweightMMM:
       degrees_seasonality: Number of degrees for the seasonality.
       frequency: Frequency of the seasonality.
       transform_function: Media transform function to use within the model.
+      weekday_seasonality: Allow daily weekday estimation.
       model: Numpyro model to use for numpyro.infer.Predictive.
       posterior_samples: Mapping of the posterior samples.
 
@@ -230,7 +253,8 @@ class LightweightMMM:
             target_data=None,
             degrees_seasonality=degrees_seasonality,
             frequency=frequency,
-            transform_function=transform_function)
+            transform_function=transform_function,
+            weekday_seasonality=weekday_seasonality)
 
   def predict(
       self,
@@ -281,7 +305,7 @@ class LightweightMMM:
         previous_extra_features = jnp.concatenate(
             arrays=[
                 self._extra_features,
-                jnp.zeros((media_gap.shape[0], extra_features.shape[1]))
+                jnp.zeros((media_gap.shape[0], *self._extra_features.shape[1:]))
             ],
             axis=0)
     else:
@@ -303,6 +327,7 @@ class LightweightMMM:
         cost_prior=jnp.array(self._total_costs),
         degrees_seasonality=self._degrees_seasonality,
         frequency=self._seasonality_frequency,
+        weekday_seasonality=self._weekday_seasonality,
         transform_function=self._model_transform_function,
         model=self._model_function,
         posterior_samples=self.trace)["mu"][:, previous_media.shape[0]:]
@@ -335,10 +360,10 @@ class LightweightMMM:
 
   def get_posterior_metrics(
       self,
-      unscaled_costs: Optional[np.ndarray] = None,
+      unscaled_costs: Optional[jnp.ndarray] = None,
       cost_scaler: Optional[preprocessing.CustomScaler] = None,
       target_scaler: Optional[preprocessing.CustomScaler] = None
-  ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+  ) -> Tuple[jnp.ndarray, jnp.ndarray]:
     """It estimates the media effect and ROI of each channel.
 
     If data was scaled prior to training then the target and costs scalers need
@@ -354,8 +379,6 @@ class LightweightMMM:
       target_scaler: Scaler that was used to scale the target before training.
 
     Returns:
-      predictions: The mean of the posterior distribution for the taget's mean
-        parameter. If target scaler is provided predictions are unscaled.
       media_effect_hat: The average media effect for each channel.
       roi_hat: The return on investment of each channel calculated as its effect
         divided by the cost.
@@ -383,18 +406,24 @@ class LightweightMMM:
       else:
         unscaled_costs = self._total_costs
 
-    predictions = self.trace["mu"].mean(axis=1)
     if target_scaler:
-      predictions = target_scaler.inverse_transform(predictions)
-    # s for samples, d for data (time dim), c for channels (media channels)
-    effect = jnp.einsum("sdc, sc -> sc", self.trace["media_transformed"],
-                        self.trace["beta_media"])
-    percent_change = effect / self._target.sum()
-
-    if target_scaler:
-      unscaled_target_sum = target_scaler.inverse_transform(self._target).sum()
+      unscaled_target = target_scaler.inverse_transform(self._target)
     else:
-      unscaled_target_sum = self._target.sum()
+      unscaled_target = self._target
 
+    if self.media.ndim == 2:
+      # s for samples, t for time, c for media channels
+      einsum_str = "stc, sc -> sc"
+      scaled_target_sum = self._target.sum()
+      unscaled_target_sum = unscaled_target.sum()
+    elif self.media.ndim == 3:
+      # s for samples, t for time, c for media channels, g for geo
+      einsum_str = "stcg, scg -> scg"
+      scaled_target_sum = self._target.sum(axis=0)
+      unscaled_target_sum = unscaled_target.sum(axis=0)
+
+    effect = jnp.einsum(einsum_str, self.trace["media_transformed"],
+                        jnp.squeeze(self.trace["beta_media"]))
+    percent_change = effect / scaled_target_sum
     roi_hat = unscaled_target_sum * percent_change / unscaled_costs
-    return predictions, percent_change, roi_hat
+    return percent_change, roi_hat
